@@ -73,7 +73,7 @@ grn_id grn_table_get_v(grn_ctx *ctx, grn_obj *table, const void *key, int key_si
                        void **value);
 grn_id grn_table_add_v(grn_ctx *ctx, grn_obj *table, const void *key, int key_size,
                        void **value, int *added);
-GRN_API grn_rc grn_table_get_info(grn_ctx *ctx, grn_obj *table, grn_obj_flags *flags,
+GRN_API grn_rc grn_table_get_info(grn_ctx *ctx, grn_obj *table, grn_table_flags *flags,
                                   grn_encoding *encoding, grn_obj **tokenizer,
                                   grn_obj **normalizer,
                                   grn_obj **token_filters);
@@ -448,6 +448,8 @@ grn_obj *grn_obj_default_set_value_hook(grn_ctx *ctx,
 
 void grn_obj_touch(grn_ctx *ctx, grn_obj *obj, grn_timeval *tv);
 
+grn_rc grn_pvector_fin(grn_ctx *ctx, grn_obj *obj);
+
 #ifdef __cplusplus
 }
 #endif
