@@ -114,11 +114,16 @@ object_equal(mrb_state *mrb, mrb_value self)
 
   object = DATA_PTR(self);
   other_object = DATA_PTR(mrb_other);
-  if (object == other_object) {
-    return mrb_true_value();
-  } else {
-    return mrb_false_value();
-  }
+  return mrb_bool_value(object == other_object);
+}
+
+static mrb_value
+object_hash(mrb_state *mrb, mrb_value self)
+{
+  grn_obj *object;
+
+  object = DATA_PTR(self);
+  return mrb_fixnum_value((mrb_int)((uint64_t)object));
 }
 
 static mrb_value
@@ -221,6 +226,16 @@ object_is_persistent(mrb_state *mrb, mrb_value self)
   return mrb_bool_value((flags & GRN_OBJ_PERSISTENT) == GRN_OBJ_PERSISTENT);
 }
 
+static mrb_value
+object_is_true(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  grn_obj *object;
+
+  object = DATA_PTR(self);
+  return mrb_bool_value(grn_obj_is_true(ctx, object));
+}
+
 void
 grn_mrb_object_init(grn_ctx *ctx)
 {
@@ -241,6 +256,8 @@ grn_mrb_object_init(grn_ctx *ctx)
   mrb_define_method(mrb, klass, "grn_inspect",
                     object_grn_inspect, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "==", object_equal, MRB_ARGS_REQ(1));
+  mrb_define_method(mrb, klass, "eql?", object_equal, MRB_ARGS_REQ(1));
+  mrb_define_method(mrb, klass, "hash", object_hash, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "close", object_close, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "remove", object_remove, MRB_ARGS_OPT(1));
 
@@ -254,6 +271,8 @@ grn_mrb_object_init(grn_ctx *ctx)
   mrb_define_method(mrb, klass, "persistent?", object_is_persistent,
                     MRB_ARGS_NONE());
 
+  mrb_define_method(mrb, klass, "true?", object_is_true, MRB_ARGS_NONE());
+
   grn_mrb_load(ctx, "index_info.rb");
 }
 #endif
