@@ -16,6 +16,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "grn.h"
+#include "grn_ctx_impl_mrb.h"
 #include "grn_proc.h"
 #include <groonga/plugin.h>
 
@@ -45,6 +46,7 @@
 
 static grn_hash *grn_plugins = NULL;
 static grn_critical_section grn_plugins_lock;
+static grn_ctx grn_plugins_ctx;
 
 #ifdef HAVE_DLFCN_H
 #  include <dlfcn.h>
@@ -97,7 +99,7 @@ grn_plugin_reference(grn_ctx *ctx, const char *filename)
   grn_plugin **plugin = NULL;
 
   CRITICAL_SECTION_ENTER(grn_plugins_lock);
-  id = grn_hash_get(&grn_gctx, grn_plugins,
+  id = grn_hash_get(&grn_plugins_ctx, grn_plugins,
                     filename, GRN_PLUGIN_KEY_SIZE(filename),
                     (void **)&plugin);
   if (plugin) {
@@ -122,7 +124,7 @@ grn_plugin_path(grn_ctx *ctx, grn_id id)
   }
 
   CRITICAL_SECTION_ENTER(grn_plugins_lock);
-  value_size = grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin);
+  value_size = grn_hash_get_value(&grn_plugins_ctx, grn_plugins, id, &plugin);
   CRITICAL_SECTION_LEAVE(grn_plugins_lock);
 
   if (!plugin) {
@@ -150,12 +152,17 @@ static grn_rc
 grn_plugin_call_init(grn_ctx *ctx, grn_id id)
 {
   grn_plugin *plugin;
-  if (!grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin)) {
+  int size;
+
+  size = grn_hash_get_value(&grn_plugins_ctx, grn_plugins, id, &plugin);
+  if (size == 0) {
     return GRN_INVALID_ARGUMENT;
   }
+
   if (plugin->init_func) {
     return plugin->init_func(ctx);
   }
+
   return GRN_SUCCESS;
 }
 
@@ -191,17 +198,26 @@ static grn_rc
 grn_plugin_call_register(grn_ctx *ctx, grn_id id)
 {
   grn_plugin *plugin;
-  if (!grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin)) {
+  int size;
+
+  CRITICAL_SECTION_ENTER(grn_plugins_lock);
+  size = grn_hash_get_value(&grn_plugins_ctx, grn_plugins, id, &plugin);
+  CRITICAL_SECTION_LEAVE(grn_plugins_lock);
+
+  if (size == 0) {
     return GRN_INVALID_ARGUMENT;
   }
+
 #ifdef GRN_WITH_MRUBY
   if (!plugin->dl) {
     return grn_plugin_call_register_mrb(ctx, id, plugin);
   }
 #endif /* GRN_WITH_MRUBY */
+
   if (plugin->register_func) {
     return plugin->register_func(ctx);
   }
+
   return GRN_SUCCESS;
 }
 
@@ -209,12 +225,17 @@ static grn_rc
 grn_plugin_call_fin(grn_ctx *ctx, grn_id id)
 {
   grn_plugin *plugin;
-  if (!grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin)) {
+  int size;
+
+  size = grn_hash_get_value(&grn_plugins_ctx, grn_plugins, id, &plugin);
+  if (size == 0) {
     return GRN_INVALID_ARGUMENT;
   }
+
   if (plugin->fin_func) {
     return plugin->fin_func(ctx);
   }
+
   return GRN_SUCCESS;
 }
 
@@ -263,23 +284,32 @@ grn_plugin_initialize(grn_ctx *ctx, grn_plugin *plugin,
 static grn_id
 grn_plugin_open_mrb(grn_ctx *ctx, const char *filename, size_t filename_size)
 {
+  grn_ctx *plugins_ctx = &grn_plugins_ctx;
   grn_id id = GRN_ID_NIL;
   grn_plugin **plugin = NULL;
 
+  grn_ctx_impl_mrb_ensure_init(ctx);
+  if (ctx->rc != GRN_SUCCESS) {
+    return GRN_ID_NIL;
+  }
+
   if (!ctx->impl->mrb.state) {
     ERR(GRN_FUNCTION_NOT_IMPLEMENTED, "mruby support isn't enabled");
     return GRN_ID_NIL;
   }
 
-  id = grn_hash_add(&grn_gctx, grn_plugins, filename, filename_size,
+  id = grn_hash_add(plugins_ctx, grn_plugins, filename, filename_size,
                     (void **)&plugin, NULL);
   if (!id) {
     return id;
   }
 
-  *plugin = GRN_GMALLOCN(grn_plugin, 1);
+  {
+    grn_ctx *ctx = plugins_ctx;
+    *plugin = GRN_MALLOCN(grn_plugin, 1);
+  }
   if (!*plugin) {
-    grn_hash_delete_by_id(&grn_gctx, grn_plugins, id, NULL);
+    grn_hash_delete_by_id(plugins_ctx, grn_plugins, id, NULL);
     return GRN_ID_NIL;
   }
 
@@ -297,6 +327,7 @@ grn_plugin_open_mrb(grn_ctx *ctx, const char *filename, size_t filename_size)
 grn_id
 grn_plugin_open(grn_ctx *ctx, const char *filename)
 {
+  grn_ctx *plugins_ctx = &grn_plugins_ctx;
   grn_id id = GRN_ID_NIL;
   grn_dl dl;
   grn_plugin **plugin = NULL;
@@ -305,7 +336,7 @@ grn_plugin_open(grn_ctx *ctx, const char *filename)
   filename_size = GRN_PLUGIN_KEY_SIZE(filename);
 
   CRITICAL_SECTION_ENTER(grn_plugins_lock);
-  if ((id = grn_hash_get(&grn_gctx, grn_plugins, filename, filename_size,
+  if ((id = grn_hash_get(plugins_ctx, grn_plugins, filename, filename_size,
                          (void **)&plugin))) {
     (*plugin)->refcount++;
     goto exit;
@@ -325,18 +356,24 @@ grn_plugin_open(grn_ctx *ctx, const char *filename)
 #endif /* GRN_WITH_MRUBY */
 
   if ((dl = grn_dl_open(filename))) {
-    if ((id = grn_hash_add(&grn_gctx, grn_plugins, filename, filename_size,
+    if ((id = grn_hash_add(plugins_ctx, grn_plugins, filename, filename_size,
                            (void **)&plugin, NULL))) {
-      *plugin = GRN_GMALLOCN(grn_plugin, 1);
+      {
+        grn_ctx *ctx = plugins_ctx;
+        *plugin = GRN_MALLOCN(grn_plugin, 1);
+      }
       if (*plugin) {
         grn_memcpy((*plugin)->path, filename, filename_size);
         if (grn_plugin_initialize(ctx, *plugin, dl, id, filename)) {
-          GRN_GFREE(*plugin);
+          {
+            grn_ctx *ctx = plugins_ctx;
+            GRN_FREE(*plugin);
+          }
           *plugin = NULL;
         }
       }
       if (!*plugin) {
-        grn_hash_delete_by_id(&grn_gctx, grn_plugins, id, NULL);
+        grn_hash_delete_by_id(plugins_ctx, grn_plugins, id, NULL);
         if (grn_dl_close(dl)) {
           /* Now, __FILE__ set in plugin is invalid. */
           ctx->errline = 0;
@@ -372,6 +409,7 @@ exit:
 grn_rc
 grn_plugin_close(grn_ctx *ctx, grn_id id)
 {
+  grn_ctx *plugins_ctx = &grn_plugins_ctx;
   grn_rc rc;
   grn_plugin *plugin;
 
@@ -380,7 +418,7 @@ grn_plugin_close(grn_ctx *ctx, grn_id id)
   }
 
   CRITICAL_SECTION_ENTER(grn_plugins_lock);
-  if (!grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin)) {
+  if (!grn_hash_get_value(plugins_ctx, grn_plugins, id, &plugin)) {
     rc = GRN_INVALID_ARGUMENT;
     goto exit;
   }
@@ -396,8 +434,11 @@ grn_plugin_close(grn_ctx *ctx, grn_id id)
       SERR("%s", label);
     }
   }
-  GRN_GFREE(plugin);
-  rc = grn_hash_delete_by_id(&grn_gctx, grn_plugins, id, NULL);
+  {
+    grn_ctx *ctx = plugins_ctx;
+    GRN_FREE(plugin);
+  }
+  rc = grn_hash_delete_by_id(plugins_ctx, grn_plugins, id, NULL);
 
 exit:
   CRITICAL_SECTION_LEAVE(grn_plugins_lock);
@@ -416,7 +457,7 @@ grn_plugin_sym(grn_ctx *ctx, grn_id id, const char *symbol)
   }
 
   CRITICAL_SECTION_ENTER(grn_plugins_lock);
-  if (!grn_hash_get_value(&grn_gctx, grn_plugins, id, &plugin)) {
+  if (!grn_hash_get_value(&grn_plugins_ctx, grn_plugins, id, &plugin)) {
     func = NULL;
     goto exit;
   }
@@ -437,9 +478,14 @@ grn_rc
 grn_plugins_init(void)
 {
   CRITICAL_SECTION_INIT(grn_plugins_lock);
-  grn_plugins = grn_hash_create(&grn_gctx, NULL, PATH_MAX, sizeof(grn_plugin *),
+  grn_ctx_init(&grn_plugins_ctx, 0);
+  grn_plugins = grn_hash_create(&grn_plugins_ctx, NULL,
+                                PATH_MAX, sizeof(grn_plugin *),
                                 GRN_OBJ_KEY_VAR_SIZE);
-  if (!grn_plugins) { return GRN_NO_MEMORY_AVAILABLE; }
+  if (!grn_plugins) {
+    grn_ctx_fin(&grn_plugins_ctx);
+    return GRN_NO_MEMORY_AVAILABLE;
+  }
   return GRN_SUCCESS;
 }
 
@@ -448,10 +494,11 @@ grn_plugins_fin(void)
 {
   grn_rc rc;
   if (!grn_plugins) { return GRN_INVALID_ARGUMENT; }
-  GRN_HASH_EACH(&grn_gctx, grn_plugins, id, NULL, NULL, NULL, {
-    grn_plugin_close(&grn_gctx, id);
+  GRN_HASH_EACH(&grn_plugins_ctx, grn_plugins, id, NULL, NULL, NULL, {
+    grn_plugin_close(&grn_plugins_ctx, id);
   });
-  rc = grn_hash_close(&grn_gctx, grn_plugins);
+  rc = grn_hash_close(&grn_plugins_ctx, grn_plugins);
+  grn_ctx_fin(&grn_plugins_ctx);
   CRITICAL_SECTION_FIN(grn_plugins_lock);
   return rc;
 }
@@ -555,11 +602,16 @@ grn_plugin_find_path_mrb(grn_ctx *ctx, const char *path, size_t path_len)
   const char *mrb_suffix;
   size_t mrb_path_len;
 
-  mrb_suffix = grn_plugin_get_ruby_suffix();
+  grn_ctx_impl_mrb_ensure_init(ctx);
+  if (ctx->rc != GRN_SUCCESS) {
+    return NULL;
+  }
+
   if (!ctx->impl->mrb.state) {
     return NULL;
   }
 
+  mrb_suffix = grn_plugin_get_ruby_suffix();
   mrb_path_len = path_len + strlen(mrb_suffix);
   if (mrb_path_len >= PATH_MAX) {
     ERR(GRN_FILENAME_TOO_LONG,
@@ -676,13 +728,6 @@ grn_plugin_find_path(grn_ctx *ctx, const char *name)
   }
 
   path_len = strlen(path);
-  found_path = grn_plugin_find_path_mrb(ctx, path, path_len);
-  if (found_path) {
-    goto exit;
-  }
-  if (ctx->rc) {
-    goto exit;
-  }
 
   found_path = grn_plugin_find_path_so(ctx, path, path_len);
   if (found_path) {
@@ -700,6 +745,14 @@ grn_plugin_find_path(grn_ctx *ctx, const char *name)
     goto exit;
   }
 
+  found_path = grn_plugin_find_path_mrb(ctx, path, path_len);
+  if (found_path) {
+    goto exit;
+  }
+  if (ctx->rc) {
+    goto exit;
+  }
+
 exit :
   GRN_API_RETURN(found_path);
 }
@@ -768,7 +821,7 @@ grn_plugin_unregister_by_path(grn_ctx *ctx, const char *path)
   GRN_API_ENTER;
 
   CRITICAL_SECTION_ENTER(grn_plugins_lock);
-  plugin_id = grn_hash_get(&grn_gctx, grn_plugins,
+  plugin_id = grn_hash_get(&grn_plugins_ctx, grn_plugins,
                            path, GRN_PLUGIN_KEY_SIZE(path),
                            NULL);
   CRITICAL_SECTION_LEAVE(grn_plugins_lock);
@@ -832,16 +885,39 @@ grn_plugin_ensure_registered(grn_ctx *ctx, grn_obj *proc)
 {
 #ifdef GRN_WITH_MRUBY
   grn_id plugin_id;
-  const char *plugin_path;
-  uint32_t key_size;
-  grn_plugin *plugin;
-  int value_size;
+  grn_plugin *plugin = NULL;
 
-  if (!ctx->impl->mrb.state) {
+  if (!(proc->header.flags & GRN_OBJ_CUSTOM_NAME)) {
     return;
   }
 
-  if (!(proc->header.flags & GRN_OBJ_CUSTOM_NAME)) {
+  plugin_id = DB_OBJ(proc)->range;
+  CRITICAL_SECTION_ENTER(grn_plugins_lock);
+  {
+    const char *value;
+    uint32_t value_size;
+    value = grn_hash_get_value_(&grn_plugins_ctx, grn_plugins, plugin_id,
+                                &value_size);
+    if (value) {
+      plugin = *((grn_plugin **)value);
+    }
+  }
+  CRITICAL_SECTION_LEAVE(grn_plugins_lock);
+
+  if (!plugin) {
+    return;
+  }
+
+  if (plugin->dl) {
+    return;
+  }
+
+  grn_ctx_impl_mrb_ensure_init(ctx);
+  if (ctx->rc != GRN_SUCCESS) {
+    return;
+  }
+
+  if (!ctx->impl->mrb.state) {
     return;
   }
 
@@ -856,23 +932,7 @@ grn_plugin_ensure_registered(grn_ctx *ctx, grn_obj *proc)
     }
   }
 
-  plugin_id = DB_OBJ(proc)->range;
-  CRITICAL_SECTION_ENTER(grn_plugins_lock);
-  plugin_path = _grn_hash_key(&grn_gctx, grn_plugins, plugin_id, &key_size);
-  if (plugin_path) {
-    value_size = grn_hash_get_value(&grn_gctx, grn_plugins, plugin_id, &plugin);
-  }
-  CRITICAL_SECTION_LEAVE(grn_plugins_lock);
-
-  if (!plugin_path) {
-    return;
-  }
-
-  if (plugin->dl) {
-    return;
-  }
-
-  ctx->impl->plugin_path = plugin_path;
+  ctx->impl->plugin_path = plugin->path;
   grn_plugin_call_register_mrb(ctx, plugin_id, plugin);
   ctx->impl->plugin_path = NULL;
 #endif /* GRN_WITH_MRUBY */
@@ -1050,6 +1110,12 @@ grn_plugin_set_error(grn_ctx *ctx, grn_log_level level, grn_rc error_code,
   }
 }
 
+void
+grn_plugin_clear_error(grn_ctx *ctx)
+{
+  ERRCLR(ctx);
+}
+
 void
 grn_plugin_backtrace(grn_ctx *ctx)
 {
